@@ -25,6 +25,15 @@ def replace_insensitive(string, target, replacement):
         return string
 
 
+def _printable(value):
+    if isinstance(value, unicode):
+        return value.encode('unicode_escape')
+    elif isinstance(value, str):
+        return value.encode('string_escape')
+    else:
+        return repr(value)
+
+
 class DebugToolbarExtension(object):
     _static_dir = os.path.realpath(
         os.path.join(os.path.dirname(__file__), 'static'))
@@ -34,8 +43,9 @@ class DebugToolbarExtension(object):
     def __init__(self, app):
         self.app = app
         self.debug_toolbars = {}
+        self.hosts = ()
 
-        if not app.debug:
+        if not app.config.get('DEBUG_TB_ENABLED', app.debug):
             return
 
         if not app.config.get('SECRET_KEY'):
@@ -45,8 +55,11 @@ class DebugToolbarExtension(object):
 
         DebugToolbar.load_panels(app)
 
+        self.hosts = app.config.get('DEBUG_TB_HOSTS', ())
+
         self.app.before_request(self.process_request)
         self.app.after_request(self.process_response)
+        self.app.teardown_request(self.teardown_request)
 
         # Monkey-patch the Flask.dispatch_request method
         app.dispatch_request = self.dispatch_request
@@ -58,6 +71,7 @@ class DebugToolbarExtension(object):
             extensions=['jinja2.ext.i18n'],
             loader=PackageLoader(__name__, 'templates'))
         self.jinja_env.filters['urlencode'] = url_quote_plus
+        self.jinja_env.filters['printable'] = _printable
 
         app.add_url_rule('/_debug_toolbar/static/<path:filename>',
             '_debug_toolbar.static', self.send_static_file)
@@ -90,6 +104,10 @@ class DebugToolbarExtension(object):
         """Return a boolean to indicate if we need to show the toolbar."""
         if request.path.startswith('/_debug_toolbar/'):
             return False
+
+        if self.hosts and request.remote_addr not in self.hosts:
+            return False
+
         return True
 
     def send_static_file(self, filename):
@@ -160,6 +178,9 @@ class DebugToolbarExtension(object):
 
         return response
 
+    def teardown_request(self, exc):
+        self.debug_toolbars.pop(request._get_current_object(), None)
+
     def render(self, template_name, context):
         template = self.jinja_env.get_template(template_name)
         return template.render(**context)
